from collections import defaultdict
from collections.abc import Generator
from enum import StrEnum
from pathlib import Path
from typing import Any

from fastapi import Query
from progress.bar import Bar
from pydantic import BaseModel
from pyimporters_plugins.base import KnowledgeParserBase, KnowledgeParserOptions, Term, maybe_archive
from rdflib import RDF, SKOS, Graph, URIRef
from rdflib.resource import Resource


class RDFFormat(StrEnum):
    xml = "xml"
    n3 = "n3"
    turtle = "turtle"
    nt = "nt"
    json_ld = "json-ld"


class SKOSOptions(KnowledgeParserOptions):
    """Options for the RDF knowledge import"""

    rdf_format: RDFFormat = Query(RDFFormat.xml, description="RDF format")
    concept_class: str = Query("http://www.w3.org/2004/02/skos/core#Concept", description="")


SKOSOptionsModel = SKOSOptions


class SKOSKnowledgeParser(KnowledgeParserBase):
    def parse(
        self, source: Path, options: KnowledgeParserOptions | dict[str, Any], bar: Bar
    ) -> Generator[Term, None, None]:
        options = SKOSOptions(**options) if isinstance(options, dict) else options
        bar.max = 100
        bar.start()
        g = Graph()
        mode = "rb" if options.rdf_format in [RDFFormat.nt, RDFFormat.json_ld] else "r"
        with maybe_archive(source, mode=mode) as file:
            bar.next(10)
            thes = g.parse(file=file, format=options.rdf_format)
            bar.next(10)
            concept_class = URIRef(options.concept_class)
            bar.max = len(list(thes.subjects(predicate=RDF.type, object=concept_class)))
            for curi in thes[: RDF.type : concept_class]:
                bar.next()
                c = Resource(g, curi)
                concept: Term = None
                for prefLabel in c.objects(SKOS.prefLabel):
                    if (
                        prefLabel.language is None
                        or options.lang == "xx"
                        or prefLabel.language.startswith(options.lang)
                    ):
                        concept: Term = Term(identifier=str(curi), preferredForm=prefLabel.value)
                if concept:
                    props = defaultdict(list)
                    for altLabel in c.objects(SKOS.altLabel):
                        if (
                            altLabel.language is None
                            or options.lang == "xx"
                            or altLabel.language.startswith(options.lang)
                        ):
                            props["altForms"].append(altLabel.value)
                    if props:
                        concept.properties = props
                    yield concept

    @classmethod
    def get_schema(cls) -> type[KnowledgeParserOptions]:
        return SKOSOptions

    @classmethod
    def get_model(cls) -> type[BaseModel]:
        return SKOSOptionsModel

    @classmethod
    def get_extensions(cls) -> list[str]:
        return ["xml", "zip", "rdf", "skos", "jsonld", "ttl", "nt", "n3"]



from pathlib import Path

from progress.bar import Bar
from pyimporters_plugins.base import Term

from pyimporters_skos.skos import RDFFormat, SKOSKnowledgeParser, SKOSOptionsModel


def check_expected(concepts):
    assert len(concepts) == 1
    c0: Term = concepts[0]
    assert c0.identifier == "http://skos.um.es/unescothes/C02796"
    assert c0.preferredForm == "Occupations"
    assert len(c0.properties["altForms"]) == 4
    assert set(c0.properties["altForms"]) == {"Professional occupations", "Jobs", "Careers", "Professions"}


def test_xml():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/test.rdf")
    parser = SKOSKnowledgeParser()
    options = SKOSOptionsModel(rdf_format=RDFFormat.xml)
    concepts = list(parser.parse(source, options.model_dump(), Bar("Processing")))
    check_expected(concepts)


def test_skos_fr():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/categories.skos")
    parser = SKOSKnowledgeParser()
    options = SKOSOptionsModel(
        rdf_format=RDFFormat.xml, lang="fr", concept_class="http://www.artemis.gouv.fr/skos#Category"
    )
    concepts = list(parser.parse(source, options.model_dump(), Bar("Processing")))
    assert len(concepts) == 27
    c0: Term = concepts[0]
    assert c0.identifier == "http://www.artemis.fr/cbd6ac32-a405-4100-9f7c-6b00a9fd2484/Marine%20marchande"
    assert c0.preferredForm == "Marine marchande"


def test_zipped_xml():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/test.rdf.zip")
    parser = SKOSKnowledgeParser()
    options = SKOSOptionsModel(rdf_format=RDFFormat.xml)
    concepts = list(parser.parse(source, options.model_dump(), Bar("Processing")))
    check_expected(concepts)


def test_n3():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/test.n3")
    parser = SKOSKnowledgeParser()
    options = SKOSOptionsModel(rdf_format=RDFFormat.n3)
    concepts = list(parser.parse(source, options.model_dump(), Bar("Processing")))
    check_expected(concepts)


def test_nt():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/test.nt")
    parser = SKOSKnowledgeParser()
    options = SKOSOptionsModel(rdf_format=RDFFormat.nt)
    concepts = list(parser.parse(source, options.model_dump(), Bar("Processing")))
    check_expected(concepts)


def test_zipped_nt():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/test.nt.zip")
    parser = SKOSKnowledgeParser()
    options = SKOSOptionsModel(rdf_format=RDFFormat.nt)
    concepts = list(parser.parse(source, options.model_dump(), Bar("Processing")))
    check_expected(concepts)


def test_jsonld():
    testdir = Path(__file__).parent
    source = Path(testdir, "data/test.jsonld")
    parser = SKOSKnowledgeParser()
    options = SKOSOptionsModel(rdf_format=RDFFormat.json_ld)
    concepts = list(parser.parse(source, options.model_dump(), Bar("Processing")))
    check_expected(concepts)
